﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Analysis;
using HeuristicLab.MainForm;

namespace HeuristicLab.DataPreprocessing.Views {

  [View("Histogram View")]
  [Content(typeof(HistogramContent), true)]
  public partial class HistogramView : PreprocessingChartView {

    public List<double> Classification { get; set; }
    public bool IsDetailedChartViewEnabled { get; set; }

    public HistogramView() {
      InitializeComponent();
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content != null) {
        classifierComboBox.Items.Clear();
        classifierComboBox.Items.Add("None");

        foreach (string var in Content.GetVariableNamesForHistogramClassification()) {
          classifierComboBox.Items.Add(var);
        }

        if (classifierComboBox.SelectedItem == null && Content.ClassifierVariableIndex < classifierComboBox.Items.Count) {
          classifierComboBox.SelectedIndex = Content.ClassifierVariableIndex;
        }
      }
    }

    protected override DataTable CreateDataTable(string variableName) {
      var dt = new DataTable();
      var row = Content.CreateDataRow(variableName, DataRowVisualProperties.DataRowChartType.Histogram);
      if (Classification == null) {
        dt.Rows.Add(row);
      } else {
        dt.VisualProperties.Title = variableName;
        //var valuesPerClass = row.Values.Zip(Classification, (value, @class) => new { value, @class })
        //                        .GroupBy(x => x.@class)
        //                        .ToDictionary(x => x.Key, x => x.Select(v => v.value));
        var valuesPerClass = row.Values.Select((i, index) => new { i, j = Classification.ToList()[index] })
                                       .GroupBy((x) => x.j)
                                       .ToDictionary(x => x.Key, x => x.Select(v => v.i)
                                       .ToList());
        foreach (var entry in valuesPerClass) {
          var classRow = new DataRow(entry.Key.ToString());
          classRow.VisualProperties.ChartType = DataRowVisualProperties.DataRowChartType.Histogram;
          classRow.Values.AddRange(entry.Value);
          dt.Rows.Add(classRow);
        }
      }
      return dt;
    }

    public new HistogramContent Content {
      get { return (HistogramContent)base.Content; }
      set { base.Content = value; }
    }

    private void classifierComboBox_SelectedIndexChanged(object sender, EventArgs e) {
      if (classifierComboBox.SelectedItem == null)
        return;

      if (classifierComboBox.SelectedIndex != 0) {
        int columndIndex = Content.PreprocessingData.GetColumnIndex(classifierComboBox.SelectedItem.ToString());
        Classification = Content.PreprocessingData.GetValues<double>(columndIndex).ToList();
      } else {
        Classification = null;
      }

      Content.ClassifierVariableIndex = classifierComboBox.SelectedIndex;

      // rebuild datatables
      InitData();
      GenerateLayout();
    }
  }
}
